import { c as baseClone } from "./graph-COFI8eKA.js";
var CLONE_SYMBOLS_FLAG = 4;
function clone(value) {
  return baseClone(value, CLONE_SYMBOLS_FLAG);
}
export {
  clone as c
};
//# sourceMappingURL=clone-C9hMUx6Q.js.map
