function initRange(domain, range) {
  switch (arguments.length) {
    case 0:
      break;
    case 1:
      this.range(domain);
      break;
    default:
      this.range(range).domain(domain);
      break;
  }
  return this;
}
export {
  initRange as i
};
//# sourceMappingURL=init-ZxktEp_H.js.map
