import { aE as Utils, aF as Color } from "./index-TCZc8GNC.js";
const channel = (color, channel2) => {
  return Utils.lang.round(Color.parse(color)[channel2]);
};
export {
  channel as c
};
//# sourceMappingURL=channel-BKfYN3Ui.js.map
