/* agi-green.css*/

body {
    font-family: 'Arial', sans-serif;
    background-color: #e7fcf3;
    margin: 0;
    padding: 0;
    display: flex;
    flex-direction: column;
    height: 100%;
}

.app-layout {
    display: flex;
    width: 100%;
    height: 100vh; /* Use viewport height to make the layout fill the entire screen */
}

.vsplitter {
    width: 4px;
    height: 100%;
    cursor: col-resize;
    background-color: #afe6da;
    border-left: 1px solid #f7fffe;
    border-right: 1px solid #618983;
}

.avatar {
    width: 40px;
    height: 40px;
    border-radius: 10px;
    margin-right: 10px;
}

#chat-pane {
    width: calc(50%-2px);
    min-width: 200px;
    overflow-y: auto; /* Allow scrolling if content exceeds the pane's height */
}

#workspace-pane {
    padding: 10px;
    background-color: #ecfef4;
    width: calc(50%-2px);
    overflow-y: auto; /* Allow scrolling if content exceeds the pane's height */
}

#messages {
    flex: 1;
    overflow-y: auto;
    border-bottom: 1px solid #e0e0e0;
}

.chat-message-block {
    padding: 8px;
}

.chat-message {
    flex: 1;
    padding: 8px;
    border-radius: 5px;
    background-color: #fff;
    box-shadow: 0 2px 6px rgba(0,0,0,0.1);
}

input[type="text"],

.chat-message-block
{
    display: flex;         /* Use flexbox for layout */
    align-items: center;   /* Vertically center the content */
}

#chat-input-area
{
    width: calc(100% - 30px);
    padding: 10px;
    position: relative;
}

#chat-input-text {
    width: 100%;
    height: 100%;
    resize: none;
    border-radius: 5px;
    position: relative;
    overflow-y: auto;
    max-height: 300px;
}


#send-button {
    position: absolute;
    bottom: 18px;
    right: 9px;
    width: 30px;
    height: 30px;
    border: none;
    border-radius: 5px;
    background-color: #037c85;
}

#send-button:hover {
    background-color: #006563;
}

#send-button img {
    width: 100%;
    height: 100%;
}


.button-selected img {
    filter: none !important;
    opacity: 1 !important;
}

th, td {
    border: 1px solid #034905;
    padding: 8px;
  }